@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2008 Red Hat, Inc.
+ * (C) Copyright 2007 - 2010 Red Hat, Inc.
  * (C) Copyright 2007 - 2008 Novell, Inc.
  */
 
@@ -86,7 +86,8 @@ typedef struct {
 	GByteArray *bssid;
 	guint32 rate;
 	guint32 tx_power;
-	GByteArray *mac_address;
+	GByteArray *device_mac_address;
+	GByteArray *cloned_mac_address;
 	guint32 mtu;
 	GSList *seen_bssids;
 	char *security;
@@ -102,6 +103,7 @@ enum {
 	PROP_RATE,
 	PROP_TX_POWER,
 	PROP_MAC_ADDRESS,
+	PROP_CLONED_MAC_ADDRESS,
 	PROP_MTU,
 	PROP_SEEN_BSSIDS,
 	PROP_SEC,
@@ -233,9 +235,6 @@ nm_setting_wireless_ap_security_compatible (NMSettingWireless *s_wireless,
 	if (   !strcmp (key_mgmt, "wpa-psk")
 	    || !strcmp (key_mgmt, "wpa-eap")) {
 
-		if (!(ap_flags & NM_802_11_AP_FLAGS_PRIVACY))
-			return FALSE;
-
 		if (!strcmp (key_mgmt, "wpa-psk")) {
 			if (   !(ap_wpa & NM_802_11_AP_SEC_KEY_MGMT_PSK)
 			    && !(ap_rsn & NM_802_11_AP_SEC_KEY_MGMT_PSK))
@@ -356,7 +355,15 @@ nm_setting_wireless_get_mac_address (NMSettingWireless *setting)
 {
 	g_return_val_if_fail (NM_IS_SETTING_WIRELESS (setting), NULL);
 
-	return NM_SETTING_WIRELESS_GET_PRIVATE (setting)->mac_address;
+	return NM_SETTING_WIRELESS_GET_PRIVATE (setting)->device_mac_address;
+}
+
+const GByteArray *
+nm_setting_wireless_get_cloned_mac_address (NMSettingWireless *setting)
+{
+	g_return_val_if_fail (NM_IS_SETTING_WIRELESS (setting), NULL);
+
+	return NM_SETTING_WIRELESS_GET_PRIVATE (setting)->cloned_mac_address;
 }
 
 guint32
@@ -483,31 +490,11 @@ verify (NMSetting *setting, GSList *all_settings, GError **error)
 	}
 
 	if (priv->channel) {
-		if (!strcmp (priv->band, "a")) {
-			int i;
-			int valid_channels[] = { 7, 8, 9, 11, 12, 16, 34, 36, 40, 44, 48,
-			                         52, 56, 60, 64, 100, 104, 108, 112, 116,
-			                         120, 124, 128, 132, 136, 140, 149, 153,
-			                         157, 161, 165, 183, 184, 185, 187, 188,
-			                         192, 196, 0 };
-
-			for (i = 0; valid_channels[i]; i++) {
-				if (priv->channel == valid_channels[i])
-					break;
-			}
-
-			if (valid_channels[i] == 0) {
-				g_set_error (error,
-				             NM_SETTING_WIRELESS_ERROR,
-				             NM_SETTING_WIRELESS_ERROR_INVALID_PROPERTY,
-				             NM_SETTING_WIRELESS_CHANNEL);
-				return FALSE;
-			}
-		} else if (!strcmp (priv->band, "bg") && priv->channel > 14) {
-				g_set_error (error,
-				             NM_SETTING_WIRELESS_ERROR,
-				             NM_SETTING_WIRELESS_ERROR_INVALID_PROPERTY,
-				             NM_SETTING_WIRELESS_CHANNEL);
+		if (!nm_utils_wifi_is_channel_valid (priv->channel, priv->band)) {
+			g_set_error (error,
+			             NM_SETTING_WIRELESS_ERROR,
+			             NM_SETTING_WIRELESS_ERROR_INVALID_PROPERTY,
+			             NM_SETTING_WIRELESS_CHANNEL);
 			return FALSE;
 		}
 	}
@@ -520,7 +507,7 @@ verify (NMSetting *setting, GSList *all_settings, GError **error)
 		return FALSE;
 	}
 
-	if (priv->mac_address && priv->mac_address->len != ETH_ALEN) {
+	if (priv->device_mac_address && priv->device_mac_address->len != ETH_ALEN) {
 		g_set_error (error,
 		             NM_SETTING_WIRELESS_ERROR,
 		             NM_SETTING_WIRELESS_ERROR_INVALID_PROPERTY,
@@ -528,6 +515,14 @@ verify (NMSetting *setting, GSList *all_settings, GError **error)
 		return FALSE;
 	}
 
+	if (priv->cloned_mac_address && priv->cloned_mac_address->len != ETH_ALEN) {
+		g_set_error (error,
+		             NM_SETTING_WIRELESS_ERROR,
+		             NM_SETTING_WIRELESS_ERROR_INVALID_PROPERTY,
+		             NM_SETTING_WIRELESS_CLONED_MAC_ADDRESS);
+		return FALSE;
+	}
+
 	for (iter = priv->seen_bssids; iter; iter = iter->next) {
 		struct ether_addr addr;
 
@@ -571,8 +566,10 @@ finalize (GObject *object)
 		g_byte_array_free (priv->ssid, TRUE);
 	if (priv->bssid)
 		g_byte_array_free (priv->bssid, TRUE);
-	if (priv->mac_address)
-		g_byte_array_free (priv->mac_address, TRUE);
+	if (priv->device_mac_address)
+		g_byte_array_free (priv->device_mac_address, TRUE);
+	if (priv->cloned_mac_address)
+		g_byte_array_free (priv->cloned_mac_address, TRUE);
 
 	nm_utils_slist_free (priv->seen_bssids, g_free);
 
@@ -614,9 +611,14 @@ set_property (GObject *object, guint prop_id,
 		priv->tx_power = g_value_get_uint (value);
 		break;
 	case PROP_MAC_ADDRESS:
-		if (priv->mac_address)
-			g_byte_array_free (priv->mac_address, TRUE);
-		priv->mac_address = g_value_dup_boxed (value);
+		if (priv->device_mac_address)
+			g_byte_array_free (priv->device_mac_address, TRUE);
+		priv->device_mac_address = g_value_dup_boxed (value);
+		break;
+	case PROP_CLONED_MAC_ADDRESS:
+		if (priv->cloned_mac_address)
+			g_byte_array_free (priv->cloned_mac_address, TRUE);
+		priv->cloned_mac_address = g_value_dup_boxed (value);
 		break;
 	case PROP_MTU:
 		priv->mtu = g_value_get_uint (value);
@@ -666,6 +668,9 @@ get_property (GObject *object, guint prop_id,
 	case PROP_MAC_ADDRESS:
 		g_value_set_boxed (value, nm_setting_wireless_get_mac_address (setting));
 		break;
+	case PROP_CLONED_MAC_ADDRESS:
+		g_value_set_boxed (value, nm_setting_wireless_get_cloned_mac_address (setting));
+		break;
 	case PROP_MTU:
 		g_value_set_uint (value, nm_setting_wireless_get_mtu (setting));
 		break;
@@ -696,91 +701,233 @@ nm_setting_wireless_class_init (NMSettingWirelessClass *setting_class)
 	parent_class->verify       = verify;
 
 	/* Properties */
+	/**
+	 * NMSettingWireless:ssid:
+	 *
+	 * SSID of the WiFi network.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_SSID,
 		 _nm_param_spec_specialized (NM_SETTING_WIRELESS_SSID,
 							   "SSID",
-							   "SSID",
+							   "SSID of the WiFi network.  Must be specified.",
 							   DBUS_TYPE_G_UCHAR_ARRAY,
 							   G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingWireless:mode:
+	 *
+	 * WiFi network mode; one of 'infrastructure' or 'adhoc'.  If blank,
+	 * infrastructure is assumed.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_MODE,
 		 g_param_spec_string (NM_SETTING_WIRELESS_MODE,
 						  "Mode",
-						  "Mode",
+						  "WiFi network mode; one of 'infrastructure' or "
+						  "'adhoc'.  If blank, infrastructure is assumed.",
 						  NULL,
 						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingWireless:band:
+	 *
+	 * 802.11 frequency band of the network.  One of 'a' for 5GHz 802.11a or
+	 * 'bg' for 2.4GHz 802.11.  This will lock associations to the WiFi network
+	 * to the specific band, i.e. if 'a' is specified, the device will not
+	 * associate with the same network in the 2.4GHz band even if the network's
+	 * settings are compatible.  This setting depends on specific driver
+	 * capability and may not work with all drivers.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_BAND,
 		 g_param_spec_string (NM_SETTING_WIRELESS_BAND,
 						  "Band",
-						  "Band",
+						  "802.11 frequency band of the network.  One of 'a' "
+						  "for 5GHz 802.11a or 'bg' for 2.4GHz 802.11.  This "
+						  "will lock associations to the WiFi network to the "
+						  "specific band, i.e. if 'a' is specified, the device "
+						  "will not associate with the same network in the "
+						  "2.4GHz band even if the network's settings are "
+						  "compatible.  This setting depends on specific driver "
+						  "capability and may not work with all drivers.",
 						  NULL,
 						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingWireless:channel:
+	 *
+	 * Wireless channel to use for the WiFi connection.  The device will only
+	 * join (or create for Ad-Hoc networks) a WiFi network on the specified
+	 * channel.  Because channel numbers overlap between bands, this property
+	 * also requires the 'band' property to be set.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_CHANNEL,
 		 g_param_spec_uint (NM_SETTING_WIRELESS_CHANNEL,
 						"Channel",
-						"Channel",
+						"Wireless channel to use for the WiFi connection.  The "
+						"device will only join (or create for Ad-Hoc networks) "
+						"a WiFi network on the specified channel.  Because "
+						"channel numbers overlap between bands, this property "
+						"also requires the 'band' property to be set.",
 						0, G_MAXUINT32, 0,
-						G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
+						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingWireless:bssid:
+	 *
+	 * If specified, directs the device to only associate with the given access
+	 * point.  This capability is highly driver dependent and not supported by
+	 * all devices.  Note: this property does not control the BSSID used when
+	 * creating an Ad-Hoc network and is unlikely to in the future.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_BSSID,
 		 _nm_param_spec_specialized (NM_SETTING_WIRELESS_BSSID,
 							   "BSSID",
-							   "BSSID",
+							   "If specified, directs the device to only associate "
+							   "with the given access point.  This capability is "
+							   "highly driver dependent and not supported by all "
+							   "devices.  Note: this property does not control "
+							   "the BSSID used when creating an Ad-Hoc network "
+							   "and is unlikely to in the future.",
 							   DBUS_TYPE_G_UCHAR_ARRAY,
 							   G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingWireless:rate:
+	 *
+	 * If non-zero, directs the device to only use the specified bitrate for
+	 * communication with the access point.  Units are in Kb/s, ie 5500 = 5.5
+	 * Mbit/s.  This property is highly driver dependent and not all devices
+	 * support setting a static bitrate.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_RATE,
 		 g_param_spec_uint (NM_SETTING_WIRELESS_RATE,
 						"Rate",
-						"Rate",
+						"If non-zero, directs the device to only use the "
+						"specified bitrate for communication with the access "
+						"point.  Units are in Kb/s, ie 5500 = 5.5 Mbit/s.  This "
+						"property is highly driver dependent and not all devices "
+						"support setting a static bitrate.",
 						0, G_MAXUINT32, 0,
-						G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
+						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
 
+	/**
+	 * NMSettingWireless:tx-power:
+	 *
+	 * If non-zero, directs the device to use the specified transmit power.
+	 * Units are dBm.  This property is highly driver dependent and not all
+	 * devices support setting a static transmit power.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_TX_POWER,
 		 g_param_spec_uint (NM_SETTING_WIRELESS_TX_POWER,
 						"TX Power",
-						"TX Power",
+						"If non-zero, directs the device to use the specified "
+						"transmit power.  Units are dBm.  This property is highly "
+						"driver dependent and not all devices support setting a "
+						"static transmit power.",
 						0, G_MAXUINT32, 0,
-						G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
+						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
 
+	/**
+	 * NMSettingWireless:mac-address:
+	 *
+	 * If specified, this connection will only apply to the WiFi device
+	 * whose permanent MAC address matches. This property does not change the MAC address
+	 * of the device (i.e. MAC spoofing).
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_MAC_ADDRESS,
 		 _nm_param_spec_specialized (NM_SETTING_WIRELESS_MAC_ADDRESS,
-							   "MAC Address",
-							   "Harware address",
+							   "Device MAC Address",
+							   "If specified, this connection will only apply to "
+							   "the WiFi device whose permanent MAC address matches.  "
+							   "This property does not change the MAC address "
+							   "of the device (i.e. MAC spoofing).",
 							   DBUS_TYPE_G_UCHAR_ARRAY,
 							   G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
 
+	/**
+	 * NMSettingWireless:cloned-mac-address:
+	 *
+	 * If specified, request that the Wifi device use this MAC address instead of its
+	 * permanent MAC address.  This is known as MAC cloning or spoofing.
+	 **/
+	g_object_class_install_property
+		(object_class, PROP_CLONED_MAC_ADDRESS,
+		 _nm_param_spec_specialized (NM_SETTING_WIRELESS_CLONED_MAC_ADDRESS,
+	                                     "Spoof MAC Address",
+	                                     "If specified, request that the WiFi device use "
+	                                     "this MAC address instead of its permanent MAC address.  "
+	                                     "This is known as MAC cloning or spoofing.",
+	                                     DBUS_TYPE_G_UCHAR_ARRAY,
+	                                     G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
+
+	/**
+	 * NMSettingWireless:seen-bssids:
+	 *
+	 * A list of BSSIDs (each BSSID formatted as a MAC address like
+	 * '00:11:22:33:44:55') that have been detected as part of the WiFI network.
+	 * The settings service will usually populate this property by periodically
+	 * asking NetworkManager what the device's current AP is while connected
+	 * to the network (or monitoring the device's 'active-ap' property) and
+	 * adding the current AP'sBSSID to this list.  This list helps NetworkManager
+	 * find hidden APs by matching up scan results with the BSSIDs in this list.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_SEEN_BSSIDS,
 		 _nm_param_spec_specialized (NM_SETTING_WIRELESS_SEEN_BSSIDS,
 							   "Seen BSSIDS",
-							   "Seen BSSIDs",
+							   "A list of BSSIDs (each BSSID formatted as a MAC "
+							   "address like '00:11:22:33:44:55') that have been "
+							   "detected as part of the WiFI network.  The "
+							   "settings service will usually populate this "
+							   "property by periodically asking NetworkManager "
+							   "what the device's current AP is while connected "
+							   "to the network (or monitoring the device's "
+							   "'active-ap' property) and adding the current AP's "
+							   "BSSID to this list.  This list helps NetworkManager "
+							   "find hidden APs by matching up scan results with "
+							   "the BSSIDs in this list.",
 							   DBUS_TYPE_G_LIST_OF_STRING,
 							   G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
 
+	/**
+	 * NMSettingWireless:mtu:
+	 *
+	 * If non-zero, only transmit packets of the specified size or smaller,
+	 * breaking larger packets up into multiple Ethernet frames.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_MTU,
 		 g_param_spec_uint (NM_SETTING_WIRELESS_MTU,
 						"MTU",
-						"MTU",
+						"If non-zero, only transmit packets of the specified "
+						"size or smaller, breaking larger packets up into "
+						"multiple Ethernet frames.",
 						0, G_MAXUINT32, 0,
-						G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
+						G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_FUZZY_IGNORE));
 
+	/**
+	 * NMSettingWireless:security:
+	 *
+	 * If the wireless connection has any security restrictions, like 802.1x,
+	 * WEP, or WPA, set this property to '802-11-wireless-security' and ensure
+	 * the connection contains a valid 802-11-wireless-security setting.
+	 **/
 	g_object_class_install_property
 		(object_class, PROP_SEC,
 		 g_param_spec_string (NM_SETTING_WIRELESS_SEC,
 						  "Security",
-						  "Security",
+						  "If the wireless connection has any security "
+						  "restrictions, like 802.1x, WEP, or WPA, set this "
+						  "property to '" NM_SETTING_WIRELESS_SECURITY_SETTING_NAME "' "
+						  "and ensure the connection contains a valid "
+						  NM_SETTING_WIRELESS_SECURITY_SETTING_NAME " setting.",
 						  NULL,
 						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
 }
